"App-level admin module so all subapps admins get registered."

from avocado.fields.admin import *
from avocado.columns.admin import *
from avocado.criteria.admin import *
from avocado.concepts.admin import *