class ValidationError(Exception):
    pass


class ConfigurationError(Exception):
    pass


class AlreadyRegisteredError(Exception):
    pass


class RegisterError(Exception):
    pass


class PermissionError(Exception):
    pass